package melmore.android;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;

import javax.net.SocketFactory;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.os.Handler;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class TranslationClient extends Activity implements SurfaceHolder.Callback {
	
	private boolean mHasSurface;
	private Bitmap mCameraImage;
	private SurfaceHolder mHolder;
	private final Handler mHandler = new Handler();
	private String mTranslation;
	private boolean mSuccess;
	
	// Create runnable for posting
    private final Runnable mUpdateResults = new Runnable() {
        public void run() {
        	showTransferStatus();
        }
    };
	
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.client);
		
		//make buttons invisible
		((Button)findViewById(R.id.retrybutton)).setVisibility(View.INVISIBLE);
		((Button)findViewById(R.id.backbutton)).setVisibility(View.INVISIBLE);
		
		//set up buttons' listeners right now
		((Button)findViewById(R.id.retrybutton)).setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		
        		startTransfer();
        		
        	}
        });
		((Button)findViewById(R.id.backbutton)).setOnClickListener(new View.OnClickListener() {
        	public void onClick(View view) {
        		
        		((Activity)view.getContext()).finish();
        		
        	}
        });
		
		//let the user know we're trying to connect
		((TextView)findViewById(R.id.networkstatus)).setText(R.string.connecting_status);
		((TextView)findViewById(R.id.networkstatus)).requestFocus();
		
		//get out the selection we want to view
		mCameraImage = (Bitmap)this.getIntent().getParcelableExtra("melmore.android.camera_image");
		
		//set up the surface view
		SurfaceView selectionView = (SurfaceView) findViewById(R.id.selectionview);
	    mHolder = selectionView.getHolder();
	    mHolder.setFixedSize(320, 240);
	    mHolder.addCallback(this);
	    
        mHasSurface = false;
		
	}
	
    public void surfaceCreated(SurfaceHolder holder) {
        // The Surface has been created
        mHasSurface = true;
        
        //draw the selection image in the middle of the canvas
        Canvas tempCanvas = mHolder.lockCanvas();
        Paint p = new Paint();
        p.setColor(Color.WHITE);
        tempCanvas.drawRect(0, 0, 319, 239, p);
        tempCanvas.drawBitmap(mCameraImage, 160 - mCameraImage.width() / 2,
        					  120 - mCameraImage.height() / 2, p);
        mHolder.unlockCanvasAndPost(tempCanvas);
        
        //let the user know we're trying to connect
		((TextView)findViewById(R.id.networkstatus)).setText(R.string.connecting_status);
        
        //start the transfer in a thread besides the UI thread
        Thread t = new Thread() {
            public void run() {
                startTransfer();
            }
        };
        t.start();
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        // Surface will be destroyed when we return. Stop editing.
        mHasSurface = false;
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int w, int h) {
        // Surface size or format has changed. This should not happen in this
        // example.
    }
    
    private void startTransfer() {
    	
		//create a socket 
		try {
			SocketFactory socketFactory = SocketFactory.getDefault();
			Socket socket = socketFactory.createSocket("boater.princeton.edu", 8800);

			//compress image to output stream and send to boater
			OutputStream os = socket.getOutputStream();
			mCameraImage.compress(Bitmap.CompressFormat.JPEG, 100, os);
			
			//read the translation
			BufferedReader br = new BufferedReader(new InputStreamReader(socket.getInputStream()));
			mTranslation = "Translation: \n";
			String temp;
			while (!(temp = br.readLine()).equals("")) {
				mTranslation += temp + "\n";
			}
			
			mSuccess = true;
			
			//close the socket
			socket.close();
			
		} catch(IOException e) {
			mSuccess = false;
		}
		
		mHandler.post(mUpdateResults);
    }
    
    private void showTransferStatus() {
    	// if transfer successful, show translation
        if (mSuccess) {
        	
			//set the translation to be the text under the image
			((TextView)findViewById(R.id.networkstatus)).setText(mTranslation);
			
			//show return button
			((Button)findViewById(R.id.backbutton)).setVisibility(View.VISIBLE);
			((Button)findViewById(R.id.backbutton)).requestFocus();
        }
        
        //otherwise, allow retry or quit
        else {
        	((TextView)findViewById(R.id.networkstatus)).setText(R.string.error_status);
			
			//show return button and retry button
			((Button)findViewById(R.id.backbutton)).setVisibility(View.VISIBLE);
			((Button)findViewById(R.id.retrybutton)).setVisibility(View.VISIBLE);
			((Button)findViewById(R.id.retrybutton)).requestFocus();
        }
    }
}
